/******************************************************************************
 *  Name:    Reference Solution
 *
 ******************************************************************************/

public class BinaryPoly {
    private int[] coef;   // coefficients p(x) = sum { coef[i] * x^i }
    private int degree;   // degree of polynomial (-1 for the zero polynomial)

    // creates a binary polynomial with coefficients[i] corresponding
    // to the coefficient of x^i
    public BinaryPoly(int[] coefficients) {
        degree = coefficients.length - 1;
        while (degree >= 0 && coefficients[degree] == 0)
            degree--;
        coef = new int[degree + 1];
        for (int i = 0; i <= degree; i++)
            coef[i] = coefficients[i];
    }

    // returns the degree of this polynomial
    public int degree() {
        return degree;
    }

    // returns the coefficient (0 or 1) of x^i in this polynomial
    public int coefAt(int i) {
        return coef[i];
    }

    // returns the result of adding this polynomial and that polynomial
    public BinaryPoly plus(BinaryPoly that) {
        int d = Math.max(this.degree, that.degree);
        int[] sumCoef = new int[1 + d];
        for (int i = 0; i <= this.degree; i++) sumCoef[i] ^= this.coef[i];
        for (int i = 0; i <= that.degree; i++) sumCoef[i] ^= that.coef[i];
        return new BinaryPoly(sumCoef);
    }

    // returns a string representation of this polynomial
    public String toString() {
        if  (degree == -1) return "0";
        StringBuilder sb = new StringBuilder();
        for (int i = degree; i >= 0; i--) {
            if (coef[i] == 0) continue;
            if (sb.length() > 0) sb.append(" + ");
            if (i == 0) sb.append("1");
            else if (i == 1) sb.append("x");
            else  sb.append("x^" + i);
        }
        return sb.toString();
    }


    // unit test
    public static void main(String[] args) {
        // p(x) = x^4 + x^3 + 1
        int[] coefficients1 = { 1, 0, 0, 1, 1 };
        BinaryPoly p = new BinaryPoly(coefficients1);
        StdOut.println(p);                  // prints x^4 + x^3 + 1

        // q(x) = x^4 + x + 1
        int[] coefficients2 = { 1, 1, 0, 0, 1 };
        BinaryPoly q = new BinaryPoly(coefficients2);
        StdOut.println(q);                   // prints x^4 + x + 1

        // call instance methods
        StdOut.println(p.degree());          // prints 4
        StdOut.println(p.plus(q));           // prints x^3 + x
        StdOut.println(p.plus(q).degree());  // prints 3
        StdOut.println(p.coefAt(0));         // prints 1
        StdOut.println(p.coefAt(1));         // prints 0
    }
}
